<?php

namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fai01000CancelamentoDeApuracao extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $crudService;
	protected $parameterBag;
	protected $environment;
	protected $connection;
	protected $entityManager;
	protected $dataSourceName = "fai01000_clienfilial_cliente_ctrlapurfat_notafiscal";

	public function __construct($crudService, $parameterBag, Environment $environment, $dataSourceManager, $entityManager, $connection) {
		$this->crudService          = $crudService;
		$this->parameterBag         = $parameterBag;
		$this->environment          = $environment;
		$this->dataSourceManager    = $dataSourceManager;
		$this->entityManager    	= $entityManager;
		$this->connection    		= $connection;
	}

	public function validaCancelamento(DTO\Request\DataSet $request, DTO\Response $response) {
		
		$rows = $request->getDataSet()->getRows();
		$invalidRows = "";
		
		foreach($rows as $row){
			$conditions = array (
				array (
					"columnName"    => "CDFILIAL",
					"operator"      => "=",
					"value"         => $row['CDFILIAL']
				),
				array (
					"columnName"    => "NRAPURACFAT",
					"operator"      => "=",
					"value"         => $row['NRAPURACFAT']
				)
			);
			
			$dataSet = $this->crudService->getFilteredDataSet('fai01000_validastatusnf', $conditions);
			if(!empty($dataSet->getRows())){
				$invalidRows .= $row['CDFILIAL'].$row['NRAPURACFAT'].'_'; 
			}
		}
		$data = array(
			"name" => $invalidRows 
		);
		$response->addDataSet(new DataSet('fai01000_validastatusnf', array($invalidRows)));
	}
	
	private function deleteDadosAdic($cdfilial, $nrapuracfat){
		$sql = "DELETE FROM DADOSADIC_APURAC WHERE CDFILIAL = :CDFILIAL AND NRAPURACFAT = :NRAPURACFAT";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRAPURACFAT" => $nrapuracfat
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
    	$query->execute($params);
	}
	
	public function cancelaApuracao(DTO\Request\DataSet $request, DTO\Response $response){
		try {
			$this->connection->beginTransaction();
			$rows = $request->getDataSet()->getRows();
			$responseDataSet = array();

			foreach ($rows as $row){
				$params = array (
					'P_CDFILIAL'    => $row['CDFILIAL'],
					'P_NRAPURACFAT' => $row['NRAPURACFAT']
				);
				$query = 'SELECT MAX(NRRETROFAT) NRRETROFAT
						    FROM CTRLAPURRETROFAT
						   WHERE CDFILIAL    = :P_CDFILIAL 
						     AND NRAPURACFAT = :P_NRAPURACFAT';
				$sql = $this->connection->prepare($query);
				$sql->execute($params);
				$temRetroativo = $sql->fetchAll();
				
				if($temRetroativo[0]["NRRETROFAT"] == null){
					$notas = $this->getNotas($row['CDFILIAL'], $row['NRAPURACFAT']);
					foreach ($notas as $nota){
						$nrlanctonf = $nota['NRLANCTONF'];
						$itensNF = $this->getItensNF($row['CDFILIAL'], $nrlanctonf);
						foreach ($itensNF as $item){
							$this->baixaAf($row, $nrlanctonf, $item, $response);
						}
						$this->deletaNf($row, $nrlanctonf, $response);
					}
					$this->updateNotaFiscal($row['CDFILIAL'], $row['NRAPURACFAT']);
					$this->updateCDFR($row['CDFILIAL'], $row['NRAPURACFAT']);
					$this->deleteEfetCliFat($row['CDFILIAL'], $row['NRAPURACFAT']);
					$this->deleteEfetCliPla($row['CDFILIAL'], $row['NRAPURACFAT']);
					$this->deleteCtrlCompensacao($row['CDFILIAL'], $row['NRAPURACFAT']);
					$this->deleteDadosAdic($row['CDFILIAL'], $row['NRAPURACFAT']);
					$this->deleteCtrlApurFat($row['CDFILIAL'], $row['NRAPURACFAT'], $row['CDCLIENTE']);
				} else{
					$responseDataSet[] = [
						'NRAPURACFAT' => $row['NRAPURACFAT'],
						'CDFILIAL'    => $row['CDFILIAL']
					];
				}		
			}
		$this->connection->commit();
		} catch (\Exception $e){
			$this->connection->rollback();
    		$response->setError(new DTO\Response\Error($e->getMessage(), 1));
    	}
	}

	private function getNotas($cdfilial, $nrapuracfat){
		$conditions = array (
			array (
				"columnName"    =>"CDFILIAL",
				"operator"      =>"=",
				"value"         => $cdfilial
			),
			array (
			  "columnName"      =>"NRAPURACFAT",
			  "operator"        => "=",
			  "value"           => $nrapuracfat
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('notafiscal', $conditions);
		$rows = $dataSet->getRows();
		return $rows;
	}

	private function getItensNF ($cdfilial, $nrlanctonf){
		$conditions = array (
			array (
				"columnName"    =>"CDFILIAL",
				"operator"      =>"=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    =>"NRLANCTONF",
				"operator"      =>"=",
				"value"         => $nrlanctonf
			),
			array (
			  "columnName"      =>"NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('itemproduto', $conditions);
		return $dataSet->getRows();
	}

	private function baixaAf ($row, $nrlanctonf, $item, $response) {
		try {

			$procedureNfBaixaAf = new StoredProcedure($this->entityManager->getConnection(), 'NF_BAIXA_AF');

			$procedureNfBaixaAf->addParam(new Param('P_CDFILIAL', 		Param::PARAM_INPUT, 	$row['CDFILIAL'], 		Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_NRLANCTONF', 	Param::PARAM_INPUT, 	$nrlanctonf, 			Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_NRSEQITPROD', 	Param::PARAM_INPUT, 	$item['NRSEQITPROD'], 	Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_IDSTATUS', 		Param::PARAM_INPUT, 	'E', 					Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_NEWCDPRODEMIS',	Param::PARAM_INPUT, 	$item['CDPRODEMIS'], 	Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_OLDCDPRODEMIS',	Param::PARAM_INPUT, 	$item['CDPRODEMIS'], 	Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_NEWCDPRODUTO',	Param::PARAM_INPUT, 	$item['CDPRODUTO'], 	Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_OLDCDPRODUTO', 	Param::PARAM_INPUT, 	$item['CDPRODUTO'], 	Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_NEWCDMARCA', 	Param::PARAM_INPUT, 	$item['CDMARCA'], 		Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_OLDCDMARCA', 	Param::PARAM_INPUT, 	$item['CDMARCA'], 		Param::PARAM_TYPE_STR));
			$procedureNfBaixaAf->addParam(new Param('P_NEWQTITEMPROD',	Param::PARAM_INPUT, 	$item['QTITEMPROD'], 	Param::PARAM_TYPE_INT));
			$procedureNfBaixaAf->addParam(new Param('P_OLDQTITEMPROD',	Param::PARAM_INPUT, 	$item['QTITEMPROD'], 	Param::PARAM_TYPE_INT));
			$procedureNfBaixaAf->addParam(new Param('P_IDERRO', 		Param::PARAM_OUTPUT,	null, 					Param::PARAM_TYPE_STR));

			$procedureNfBaixaAf->execute();

		} catch (\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	private function deletaNf ($row, $nrlanctonf, $response) {
		try {
			$procedure = new StoredProcedure($this->entityManager->getConnection(), 'DELETA_NF');
			$procedure->addParam(new Param('P_FILIAL', 		Param::PARAM_INPUT, 	$row['CDFILIAL'], 		Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_LANCTONF', 	Param::PARAM_INPUT, 	$nrlanctonf, 	        Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDCORPITEM',	Param::PARAM_INPUT, 	null, 					Param::PARAM_TYPE_STR));
			$procedure->execute();

		} catch (\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	private function updateNotaFiscal ($cdfilial, $nrapuracfat) {
		$conditions = array (
			array (
				"columnName"    =>"CDFILIAL",
				"operator"      =>"=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    =>"NRAPFATUTNF",
				"operator"      =>"=",
				"value"         => $nrapuracfat
			),
			array (
			  "columnName"      =>"NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('notafiscal', $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			foreach ($rows as $row) {
				$row['CDFILIAPFATU']= null;
				$row['NRAPFATUTNF'] = null;
				$row['NRMRDPAPUR'] 	= null;
				$row['__is_new'] 	= false;
			}

			$newDataSet = new DataSet(null, $rows);
			$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
			$rowsToSave = $this->crudService->getRowsToSave($newRequest,'notafiscal');
			$this->crudService->save('notafiscal', $rowsToSave);
		}
	}

	private function updateCDFR ($cdfilial, $nrapuracfat) {
		$conditions = array (
			array (
				"columnName"    =>"CDFILIALCENT",
				"operator"      =>"=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    =>"NRAPURACFATU",
				"operator"      =>"=",
				"value"         => $nrapuracfat
			),
			array (
			  "columnName"      =>"NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('ctrlapcdfr', $conditions);
		$rows = $dataSet->getRows();

		if (!empty($rows)) {
			foreach ($rows as $row) {
				$row['CDFILIALCENT']= null;
				$row['NRAPURACFATU']= null;
				$row['IDSITUCDFR'] 	= 'C';
				$row['__is_new'] 	= false;
			}

			$newDataSet = new DataSet(null, $rows);
			$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
			$rowsToSave = $this->crudService->getRowsToSave($newRequest,'ctrlapcdfr');
			$this->crudService->save('ctrlapcdfr', $rowsToSave);
		}
	}

	private function deleteEfetCliFat ($cdfilial, $nrapuracfat){
		$conditions = array (
			array (
				"columnName"    =>"CDFILIFATU",
				"operator"      =>"=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    =>"NRAPURACFAT",
				"operator"      =>"=",
				"value"         => $nrapuracfat
			),
			array (
			  "columnName"      =>"NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('efeclifat', $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			for ($i = 0; $i < sizeof($rows); $i++) {
				$rows[$i]['CDFILIFATU']  = null;
				$rows[$i]['NRAPURACFAT'] = null;
				$rows[$i]['__is_new'] 	= false;
			}
			$newDataSet = new DataSet(null, $rows);
			$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
			$rowsToSave = $this->crudService->getRowsToSave($newRequest,'efeclifat');
			$this->crudService->save('efeclifat', $rowsToSave);
		}
	}

	private function deleteEfetCliPla ($cdfilial, $nrapuracfat){
		$conditions = array (
			array (
				"columnName"    =>"CDFILIFATU",
				"operator"      =>"=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    =>"NRAPURACFAT",
				"operator"      =>"=",
				"value"         => $nrapuracfat
			),
			array (
			  "columnName"      =>"NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('efetclie', $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			foreach ($rows as &$row) {
				$row['CDFILIFATU']= null;
				$row['NRAPURACFAT']= null;
				$row['__is_new'] 	= false;
			}
			$newDataSet = new DataSet(null, $rows);
			$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
			$rowsToSave = $this->crudService->getRowsToSave($newRequest,'efetclie');
			$this->crudService->save('efetclie', $rowsToSave);
		}
	}

	private function deleteCtrlCompensacao($cdfilial, $nrapuracfat) {
		$conditions = array (
			array (
				"columnName"    => "CDFILIAL",
				"operator"      => "=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    => "NRAPURACCOMP",
				"operator"      => "=",
				"value"         => $nrapuracfat
			),
			array (
			  "columnName"      => "NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('ctrlcompensacao', $conditions);
		$this->dataSourceManager->delete($dataSet);
	}

	private function deleteCtrlApurFat ($cdfilial, $nrapuracfat, $cdcliente) {
		$conditions = array (
			array (
				"columnName"    => "CDFILIAL",
				"operator"      => "=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    => "NRAPURACFAT",
				"operator"      => "=",
				"value"         => $nrapuracfat
			),
			array (
			  "columnName"      => "CDCLIENTE",
			  "operator"        => "=",
			  "value"           => $cdcliente
			),
			array (
			  "columnName"      => "NRORG",
			  "operator"        => "=",
			  "value"           => $this->environment->getNrOrg()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('ctrlapurfat', $conditions);
		$rows = $dataSet->getRows();
		if (count($rows) > 0) {
			$row = $rows[0];
			if(isset($row['CDTIPOFATUCT']) && $row['CDTIPOFATUCT'] == '001') {
				$newDataSet = new DataSet('ctrlapurfat', array($row));
				$this->dataSourceManager->delete($newDataSet);
			}
		}
	}
} 