<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
// use \Teknisa\GER\Services\CrudService;
// use \Teknisa\GER\Utilities\Messages;

class Fai00400EfetivosFaturamento extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'turno_efetivo';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
    protected $connection;
    protected $efetivosService;
    protected $novoCodigo;
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $connection, $efetivosService, $novoCodigo) {
        parent::__construct($crudService, $parameterBag);
        $this->entityManager     = $entityManager;
        $this->environment       = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection        = $connection;
        $this->efetivosService   = $efetivosService;
        $this->novoCodigo        = $novoCodigo;
    }
    
    public function eraseTurnos(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $turnos = array();
        foreach($rows as $row) {
            $conditions = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator"   => "=",
                    "value"      => $row['CDFILIAL']
                ),
                array(
                    "columnName" => "CDCLIENTE",
                    "operator"   => "=",
                    "value"      => $row['CDCLIENTE']
                ),
                array(
                    "columnName" => "DTFATURAMTUR",
                    "operator"   => "=",
                    "value"      => $row['DTFATURAM']
                ),
                array(
                    "columnName" => "CDSERVFILI",
                    "operator"   => "=",
                    "value"      => $row['CDSERVFILI']
                )
            );
            $filterCriteria = new FilterCriteria('turno_efetivo', $conditions);
            $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE", 'DTFATURAMTUR', "CDSERVFILI");
            $newFilter = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilter);
            $efetivosTurnos = $dataSet->getRows();
            foreach($efetivosTurnos as $turno) {
                if(isset($turno['QTCOMFATUTUR'])) {
                    $turno['CDCLIENTE'] = $row['CDCLIENTE'];
                    $turno['DTFATURAMTUR'] = $row['DTFATURAM'];
                    $turno['__is_new'] = false;
                    $turno['QTCOMFATUTUR'] = 0;
                    $turnos[] = $turno;
                }
            }
        }
        $this->crudService->save('turno_efetivo', $turnos);
    }
    
    public function validaCadastro($rows, $response) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"   => "=",
                "value"      => $rows[0]['CDFILIAL']
            )
        );
        $filterCriteria = new FilterCriteria('fai00400_paramfilial', $conditions);
        $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE", 'DTFATURAMINI', "DTFATURAMFIM");
        $newFilter = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilter);
        $result = $dataSet->getRows()[0];
        if($result['IDUTILDTPROC']==='S' && $result['IDBLQEFEDIAFECH']==='S' && $rows[0]['DTFATURAM']<($result['DTPROCESSA'] - $result['NRDIAANTDTPROC'])) {
            return $response->setError(new DTO\Response\Error("Data bloqueada para alteração de efetivos para faturamento para dias fechados.", 1));
        }
        
        $conditions[] = array(
            "columnName" => "CDCLIENTE",
            "operator"   => "=",
            "value"      => $rows[0]['CDCLIENTE']
        );
        $conditions[] = array(
            "columnName" => "DTFATURAMINI",
            "operator"   => "=",
            "value"      => $rows[0]['DTFATURAM']
        );
        $conditions[] = array(
            "columnName" => "DTFATURAMFIM",
            "operator"   => "=",
            "value"      => $rows[sizeof($rows)-1]['DTFATURAM']
        );
        $filterCriteria = new FilterCriteria('apuracao_cliente_data', $conditions);
        $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE", 'DTFATURAMINI', "DTFATURAMFIM");
        $newFilter = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilter);
        $result = $dataSet->getRows();
        if(sizeof($result)) {
            return $response->setError(new DTO\Response\Error("Efetivos do cliente já apurados na data selecionada.", 1));
        }
        return true;
    }
    
    public function cadastroAutomatico(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $isValid = $this->validaCadastro($rows, $response); 
        if($isValid !== true) {
            return $isValid;
        } else {
            try{
                $this->dataSourceName = 'efetivos_faturamento';
                $rowsToSave = array();
                foreach($rows as $i=>$row) {
                    $conditions = array(
                        array(
                            "columnName" => "CDFILIAL",
                            "operator"   => "=",
                            "value"      => $row['CDFILIAL']
                        ),
                        array(
                            "columnName" => "CDCLIENTE",
                            "operator"   => "=",
                            "value"      => $row['CDCLIENTE']
                        ),
                        array(
                            "columnName" => "DTFATURAM",
                            "operator"   => "=",
                            "value"      => $row['DTFATURAM']
                        ),
                        array(
                            "columnName" => "CDSERVFILI",
                            "operator"   => "=",
                            "value"      => $row['CDSERVFILI']
                        )
                    );
                    $filterCriteria = new FilterCriteria($this->dataSourceName, $conditions); // efetivos_faturamento
                    $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE", 'DTFATURAM', "CDSERVFILI");
                    $newFilter = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
                    $dataSet = $this->dataSourceManager->findBy($newFilter);
                    $turnos = $dataSet->getRows();
                    
                    if(!empty($turnos)){
                        $efetivos[] = $dataSet->getRows()[0];
                        $row['CDFILIFATU'] = $efetivos[$i]['CDFILIFATU'];
                        $row['CDSERVPLAN'] = $efetivos[$i]['CDSERVPLAN'] ? $efetivos[$i]['CDSERVPLAN'] : $efetivos[$i]['CDSERVFILI'];
                        $row['NRAPURACFAT'] = $efetivos[$i]['NRAPURACFAT'];
                        $row['VRSERVFATUDT'] = $efetivos[$i]['VRSERVFATUDT'];
                        if(isset($row['QTCOMEFATUFER'])||isset($row['QTCOMEFATUFER'])) {
                            $filterFeriado = array(
                                array(
                                    "columnName" => "CDFILIAL",
                                    "operator"   => "=",
                                    "value"      => $row['CDFILIAL']
                                ),
                                array(
                                    "columnName" => "DTFERICARD",
                                    "operator"   => "=",
                                    "value"      => $row['DTFATURAM']
                                )
                            );
                            $dataSet = $this->crudService->getFilteredDataSet('fericard', $filterFeriado);
                            $filterFeriado = array(
                                array(
                                    "columnName" => "CDFILIAL",
                                    "operator"   => "=",
                                    "value"      => $row['CDFILIAL']
                                ),
                                array(
                                    "columnName" => "DTFERIFILI",
                                    "operator"   => "=",
                                    "value"      => $row['DTFATURAM']
                                )
                            );
                            $dataSet2 = $this->crudService->getFilteredDataSet('ferifili', $filterFeriado);
                            if(sizeof($dataSet->getRows())||sizeof($dataSet->getRows())) {
                                $row['QTCOMEFATU'] = isset($row['QTCOMEFATUFER']) ? $row['QTCOMEFATUFER'] : $row['QTCOMEFATU'];
                                $row['QTCOMEPREV'] = isset($row['QTCOMEPREVFER']) ? $row['QTCOMEPREVFER'] : $row['QTCOMEPREV'];
                            }
                        }
                        $row['__is_new'] = true;
                        // if(isset($efetivos[$i]['QTCOMEFATU'])||isset($efetivos[$i]['QTCOMEPREV'])) {
                        //     $row['__is_new'] = false;
                        // } else {
                            
                        // }
                        try{
                            $this->crudService->save('efetivos_faturamento', array($row));
                        }catch(\Exception $e){
                            if(!strpos($e->getMessage(), "ORA-00001")){
                                throw $e;
                            }
                        }
                    }
                }
                
                $this->eraseTurnos($request, $response);
               
            }catch(\Exception $e){
                throw $e;
            }
        }
    }

    public function insertItratefetclie($itemRateio, $row) {
        
        $itratefeclifat = array();
        $itratefeclifat["CDFILIAL"]        = $row["CDFILIAL"];
        $itratefeclifat["CDSERVFILI"]      = $row["CDSERVFILI"];
        $itratefeclifat["CDSERVPLAN"]      = $row["CDSERVPLAN"];
        $itratefeclifat["CDCLIENTE"]       = $row["CDCLIENTE"];
        $itratefeclifat["DTFATURAM"]      = $row["DTFATURAM"];
       
        
        $itratefeclifat["__is_new"]        = true;

        $itratefeclifat['__is_new_code']       = true;
	    $itratefeclifat['__new_code_pk']       = $itemRateio['__new_code_pk'];
	    $itratefeclifat['__table_pk']          = $itemRateio['__table_pk'];
	    $itratefeclifat['__new_code_length']   = $itemRateio['__new_code_length'];
        $itratefeclifat["NRITRATEFECLIFAT"]     = $itemRateio["NRITRATEFECLIFAT"];
        $itratefeclifat["NRITEMRATEIOEFET"]    = $itemRateio["NRITEMRATEIOEFET"];
        $itratefeclifat["QTEFETIVOS"]          = $itemRateio["QTEFETIVOS"];

        $this->efetivosService->savePub(array($itratefeclifat), "itratefeclifat");
    }
    
    
    public function saveData($request, $response) {
        $rows = $request->getDataSet()->getRows();
        foreach($rows as $row){
            $this->entityManager->beginTransaction();
            try{
                $conditions = [
                    array(
                        "columnName" => "CDFILIAL",
                        "operator"   => '=',
                        "value"      => $row["CDFILIAL"]
                    ),
                    array(
                        "columnName" => "CDCLIENTE",
                        "operator"   => '=',
                        "value"      => $row["CDCLIENTE"]
                    ),
                    array(
                        "columnName" => "CDSERVFILI",
                        "operator"   => '=',
                        "value"      => $row["CDSERVFILI"]
                    ),
                    array(
                        "columnName" => "CDSERVPLAN",
                        "operator"   => '=',
                        "value"      => $row["CDSERVPLAN"]
                    ),
                    array(
                        "columnName" => "DTFATURAM",
                        "operator"   => '=',
                        "value"      => $row["DTFATURAM"]
                    )
                ];
                $existe = $this->crudService->getFilteredDataSet('efetivos_faturamento', $conditions);
                $result = array();
                $row['__is_new'] = sizeof($existe->getRows()) == 0 ? true : false;
                $row['changed'] = true;
                $rowsToSave = $this->crudService->getRowsToSave($request, 'efeclifat');
                $this->crudService->save('efeclifat', $rowsToSave);
                $this->entityManager->commit();
            }catch(\Exception $e){
                $this->entityManager->rollback();
                $result = array(false);
                if($e->getCode() == '1'){
                    $response->addDataSet(new DataSet ('callback', $result));
                }else{
                    throw $e;
                }
            }
        if(isset($row['rateioDataSource'])){
                if($row['EFETIVOSRATEIO'] != null){
                    $this->deleteItratefetclie($row);
                    foreach($row['rateioDataSource'] as $itemRateio){
                        if($itemRateio['QTEFETIVOS'] != null){
                            $this->insertItratefetclie($itemRateio, $row);
                        }
                    }
                }else{
                    $this->deleteItratefetclie($row);
                }
            }
        }
    }

    public function deleteItratefetclie($row) {
        $sql = "DELETE FROM 
                    ITRATEFECLIFAT 
                WHERE
                    CDFILIAL   = :CDFILIAL   AND
                    CDSERVFILI = :CDSERVFILI AND
                    CDSERVPLAN = :CDSERVPLAN AND
                    DTFATURAM = :DTFATURAM AND
                    CDCLIENTE  = :CDCLIENTE
                ";
        $params = array(
            "CDFILIAL"   => $row["CDFILIAL"],
            "CDSERVFILI" => $row["CDSERVFILI"],
            "CDSERVPLAN" => $row["CDSERVPLAN"],
            "DTFATURAM" => $row["DTFATURAM"],
            "CDCLIENTE"  => $row["CDCLIENTE"]

        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function retornaServicosCad(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $sql = "SELECT
                    COUNT(*) AS COUNTSERV
                FROM
                    EFECLIFAT
                WHERE
                    CDFILIAL = :CDFILIAL
                    AND CDSERVFILI = :CDSERVFILI
                    AND CDCLIENTE = :CDCLIENTE
                    AND DTFATURAM BETWEEN :DTINICIO AND :DTFINAL";
        
        $params = array(
            "CDFILIAL"   => $row["CDFILIAL"],
            "CDSERVFILI" => $row["CDSERVFILI"],
            "CDCLIENTE"  => $row["CDCLIENTE"],
            "DTINICIO"   => $row["DTINICIO"],
            "DTFINAL"    => $row["DTFINAL"]
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params); 
        $response->addDataSet(new DataSet('result', $stmt->fetchAll()));
    }
    
    public function apagaTurnosServicos($request, $response) { 
        $rows = $request->getDataSet()->getRows();
        $dataList = $rows[0];
        $faturamentos = $rows[1];
        $result = array();
        
        if(sizeof($faturamentos) > 0) {
            $nrlista = $this->insertLista($dataList);
            
            $sql = "SELECT
                        LI.CDLISTA AS DATA
                    FROM
                        CTRLAPURFAT CA,
                        CLIENFILIAL CF,
                        LISTA LI
                    WHERE
                        (
                            CA.CDFILIAPUR =:CDFILIAL
                        ) AND (
                            CA.CDFILIAPUR = CF.CDFILIAL
                        ) AND (
                            CA.CDCLIENTE =:CDCLIENTE
                        ) AND (
                            CA.CDCLIENTE = CF.CDCLIENTE
                        ) AND (
                            CF.IDUTILADIANT = 'N'
                        ) AND (
                            LI.NRLISTA =:NRLISTA
                        ) AND ( LI.CDLISTA BETWEEN DTINIAPURAC AND DTFIMAPURAC
                            OR
                                DTINIAPURAC BETWEEN LI.CDLISTA AND LI.CDLISTA
                            OR
                                DTFIMAPURAC BETWEEN LI.CDLISTA AND LI.CDLISTA )";
            
            $params = array(
                "CDFILIAL"   => $faturamentos[0]["CDFILIAL"],
                "CDCLIENTE"  => $faturamentos[0]["CDCLIENTE"],
                "NRLISTA"    => $nrlista
            );
            $stmt = $this->connection->prepare($sql);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
        }
        $response->addDataSet(new DataSet ('invalidDates', $result));
    }
    
    //insere lista no banco
    private function insertLista($rows){
        $nrorg = $this->environment->getNrOrg();
        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);;
        $registros = array();
        foreach ($rows as $row) {
            if(!empty($row)){
                $row = array(
                    "NRLISTA"  => $nrlista,
                    "CDLISTA"  => $row,
                    "__is_new" => true,
                    "NRORG"    => $nrorg
                );
                array_push($registros, $row);
            }
        }
        $dataSourceName = 'lista';
        $dataSet = new DataSet($dataSourceName, $registros);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rowsToSave);
        
        return $nrlista;
	}
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
}